@@ -1102,7 +1102,7 @@ static void on_core_error(void *data, uint32_t id, int seq, int res, const char
 {
 	snd_ctl_pipewire_t *ctl = data;
 
-	pw_log_error(NAME" %p: error id:%u seq:%d res:%d (%s): %s", ctl,
+	pw_log_warn(NAME" %p: error id:%u seq:%d res:%d (%s): %s", ctl,
 			id, seq, res, spa_strerror(res), message);
 
 	if (id == PW_ID_CORE) {
