@@ -39,18 +39,19 @@
 
 #define NAME "alsa-seq"
 
+#include "dll.h"
 #include "alsa-seq.h"
 
-#define CHECK(s,msg) if ((res = (s)) < 0) { spa_log_error(state->log, msg ": %s", snd_strerror(res)); return res; }
+#define CHECK(s,msg,...) if ((res = (s)) < 0) { spa_log_error(state->log, msg ": %s", ##__VA_ARGS__, snd_strerror(res)); return res; }
 
-static int seq_open(struct seq_state *state, struct seq_conn *conn)
+static int seq_open(struct seq_state *state, struct seq_conn *conn, bool with_queue)
 {
 	struct props *props = &state->props;
 	struct pollfd pfd;
         snd_seq_port_info_t *pinfo;
 	int res;
 
-	spa_log_info(state->log, "%p: ALSA seq open '%s' duplex", state, props->device);
+	spa_log_debug(state->log, "%p: ALSA seq open '%s' duplex", state, props->device);
 
 	if ((res = snd_seq_open(&conn->hndl,
 			   props->device,
@@ -68,11 +69,15 @@ static int seq_open(struct seq_state *state, struct seq_conn *conn)
 	conn->addr.client = res;
 
 	/* queue */
-	if ((res = snd_seq_alloc_queue(conn->hndl)) < 0) {
-		spa_log_error(state->log, "failed to create queue: %d", res);
-		goto error_exit_close;
+	if (with_queue) {
+		if ((res = snd_seq_alloc_queue(conn->hndl)) < 0) {
+			spa_log_error(state->log, "failed to create queue: %d", res);
+			goto error_exit_close;
+		}
+		conn->queue_id = res;
+	} else {
+		conn->queue_id = -1;
 	}
-	conn->queue_id = res;
 
 	if ((res = snd_seq_nonblock(conn->hndl, 1)) < 0)
 		spa_log_warn(state->log, "can't set nonblock mode: %s", snd_strerror(res));
@@ -86,7 +91,8 @@ static int seq_open(struct seq_state *state, struct seq_conn *conn)
         /* Enable timestamping for events sent by external subscribers. */
         snd_seq_port_info_set_timestamping(pinfo, 1);
         snd_seq_port_info_set_timestamp_real(pinfo, 1);
-        snd_seq_port_info_set_timestamp_queue(pinfo, conn->queue_id);
+	if (with_queue)
+	        snd_seq_port_info_set_timestamp_queue(pinfo, conn->queue_id);
 
         if ((res = snd_seq_create_port(conn->hndl, pinfo)) < 0) {
 		spa_log_error(state->log, "failed to create port: %s", snd_strerror(res));
@@ -111,35 +117,13 @@ error_exit_close:
 static int seq_close(struct seq_state *state, struct seq_conn *conn)
 {
 	int res;
-	spa_log_info(state->log, "%p: Device '%s' closing", state, state->props.device);
+	spa_log_debug(state->log, "%p: Device '%s' closing", state, state->props.device);
 	if ((res = snd_seq_close(conn->hndl)) < 0) {
 		spa_log_warn(state->log, "close failed: %s", snd_strerror(res));
 	}
 	return res;
 }
 
-static int seq_start(struct seq_state *state, struct seq_conn *conn)
-{
-	int res;
-	if ((res = snd_seq_start_queue(conn->hndl, conn->queue_id, NULL)) < 0) {
-		spa_log_warn(state->log, "failed to start queue: %s", snd_strerror(res));
-	}
-	while (snd_seq_drain_output(conn->hndl) > 0)
-		sleep(1);
-	return res;
-}
-
-static int seq_stop(struct seq_state *state, struct seq_conn *conn)
-{
-	int res;
-	if ((res = snd_seq_stop_queue(conn->hndl, conn->queue_id, NULL)) < 0) {
-		spa_log_warn(state->log, "failed to stop queue: %s", snd_strerror(res));
-	}
-	while (snd_seq_drain_output(conn->hndl) > 0)
-		sleep(1);
-	return res;
-}
-
 static int init_stream(struct seq_state *state, enum spa_direction direction)
 {
 	struct seq_stream *stream = &state->streams[direction];
@@ -154,6 +138,13 @@ static int init_stream(struct seq_state *state, enum spa_direction direction)
 	return 0;
 }
 
+static int uninit_stream(struct seq_state *state, enum spa_direction direction)
+{
+	struct seq_stream *stream = &state->streams[direction];
+	snd_midi_event_free(stream->codec);
+	return 0;
+}
+
 static void init_ports(struct seq_state *state)
 {
 	snd_seq_addr_t addr;
@@ -183,6 +174,9 @@ static void init_ports(struct seq_state *state)
 
 static void debug_event(struct seq_state *state, snd_seq_event_t *ev)
 {
+	if (SPA_LIKELY(!spa_log_level_enabled(state->log, SPA_LOG_LEVEL_TRACE)))
+		return;
+
 	spa_log_trace(state->log, "event type:%d flags:0x%x", ev->type, ev->flags);
 	switch (ev->flags & SND_SEQ_TIME_STAMP_MASK) {
 	case SND_SEQ_TIME_STAMP_TICK:
@@ -208,7 +202,7 @@ static void alsa_seq_on_sys(struct spa_source *source)
 	snd_seq_event_t *ev;
 	int res;
 
-	while (snd_seq_event_input (state->sys.hndl, &ev) > 0) {
+	while (snd_seq_event_input(state->sys.hndl, &ev) > 0) {
 		const snd_seq_addr_t *addr = &ev->data.addr;
 
 		if (addr->client == state->event.addr.client)
@@ -219,9 +213,10 @@ static void alsa_seq_on_sys(struct spa_source *source)
 		switch (ev->type) {
 		case SND_SEQ_EVENT_CLIENT_START:
 		case SND_SEQ_EVENT_CLIENT_CHANGE:
-			spa_log_debug(state->log, "client add/change %d", addr->client);
+			spa_log_info(state->log, "client add/change %d", addr->client);
 			break;
 		case SND_SEQ_EVENT_CLIENT_EXIT:
+			spa_log_info(state->log, "client exit %d", addr->client);
 			break;
 
 		case SND_SEQ_EVENT_PORT_START:
@@ -236,14 +231,14 @@ static void alsa_seq_on_sys(struct spa_source *source)
 				spa_log_warn(state->log, "can't get port info %d.%d: %s",
 						addr->client, addr->port, snd_strerror(res));
 			} else {
-				spa_log_debug(state->log, "port add/change %d:%d",
+				spa_log_info(state->log, "port add/change %d:%d",
 						addr->client, addr->port);
 				state->port_info(state->port_info_data, addr, info);
 			}
 			break;
 		}
 		case SND_SEQ_EVENT_PORT_EXIT:
-			spa_log_debug(state->log, "port_event: del %d:%d",
+			spa_log_info(state->log, "port_event: del %d:%d",
 					addr->client, addr->port);
 			state->port_info(state->port_info_data, addr, NULL);
 			break;
@@ -265,12 +260,12 @@ int spa_alsa_seq_open(struct seq_state *state)
 	init_stream(state, SPA_DIRECTION_INPUT);
 	init_stream(state, SPA_DIRECTION_OUTPUT);
 
-	if ((res = seq_open(state, &state->sys)) < 0)
+	if ((res = seq_open(state, &state->sys, false)) < 0)
 		return res;
 
 	snd_seq_set_client_name(state->sys.hndl, "PipeWire-System");
 
-	if ((res = seq_open(state, &state->event)) < 0)
+	if ((res = seq_open(state, &state->event, true)) < 0)
 		goto error_close_sys;
 
 	snd_seq_set_client_name(state->event.hndl, "PipeWire-RT-Event");
@@ -281,9 +276,6 @@ int spa_alsa_seq_open(struct seq_state *state)
 	addr.port = SND_SEQ_PORT_SYSTEM_ANNOUNCE;
 	snd_seq_port_subscribe_set_sender(sub, &addr);
 	snd_seq_port_subscribe_set_dest(sub, &state->sys.addr);
-	snd_seq_port_subscribe_set_queue(sub, state->sys.queue_id);
-	snd_seq_port_subscribe_set_time_update(sub, 1);
-	snd_seq_port_subscribe_set_time_real(sub, 1);
 	if ((res = snd_seq_subscribe_port(state->sys.hndl, sub)) < 0) {
 		spa_log_warn(state->log, "failed to connect announce port: %s", snd_strerror(res));
 	}
@@ -299,9 +291,7 @@ int spa_alsa_seq_open(struct seq_state *state)
 	state->sys.source.data = state;
 	spa_loop_add_source(state->main_loop, &state->sys.source);
 
-	seq_start(state, &state->sys);
-
-	/* increase queue timer resolution */
+	/* increase event queue timer resolution */
 	snd_seq_queue_timer_alloca(&timer);
 	if ((res = snd_seq_get_queue_timer(state->event.hndl, state->event.queue_id, timer)) < 0) {
 		spa_log_warn(state->log, "failed to get queue timer: %s", snd_strerror(res));
@@ -337,12 +327,14 @@ int spa_alsa_seq_close(struct seq_state *state)
 	if (!state->opened)
 		return 0;
 
-	seq_stop(state, &state->sys);
 	spa_loop_remove_source(state->main_loop, &state->sys.source);
 
 	seq_close(state, &state->sys);
 	seq_close(state, &state->event);
 
+	uninit_stream(state, SPA_DIRECTION_INPUT);
+	uninit_stream(state, SPA_DIRECTION_OUTPUT);
+
 	spa_system_close(state->data_system, state->timerfd);
 	state->opened = false;
 
@@ -366,7 +358,7 @@ static struct seq_port *find_port(struct seq_state *state,
 		struct seq_stream *stream, const snd_seq_addr_t *addr)
 {
 	uint32_t i;
-	for (i = 0; i < MAX_PORTS; i++) {
+	for (i = 0; i < stream->last_port; i++) {
 		struct seq_port *port = &stream->ports[i];
 		if (port->valid &&
 		    port->addr.client == addr->client &&
@@ -376,6 +368,53 @@ static struct seq_port *find_port(struct seq_state *state,
 	return NULL;
 }
 
+int spa_alsa_seq_activate_port(struct seq_state *state, struct seq_port *port, bool active)
+{
+	int res;
+	snd_seq_port_subscribe_t* sub;
+
+	spa_log_debug(state->log, "activate: %d.%d: started:%d active:%d wanted:%d",
+			port->addr.client, port->addr.port, state->started, port->active, active);
+
+	if (active && !state->started)
+		return 0;
+	if (port->active == active)
+		return 0;
+
+	snd_seq_port_subscribe_alloca(&sub);
+	if (port->direction == SPA_DIRECTION_OUTPUT) {
+		snd_seq_port_subscribe_set_sender(sub, &port->addr);
+		snd_seq_port_subscribe_set_dest(sub, &state->event.addr);
+	} else {
+		snd_seq_port_subscribe_set_sender(sub, &state->event.addr);
+		snd_seq_port_subscribe_set_dest(sub, &port->addr);
+	}
+
+	if (active) {
+		snd_seq_port_subscribe_set_time_update(sub, 1);
+		snd_seq_port_subscribe_set_time_real(sub, 1);
+		snd_seq_port_subscribe_set_queue(sub, state->event.queue_id);
+		if ((res = snd_seq_subscribe_port(state->event.hndl, sub)) < 0) {
+			spa_log_error(state->log, "can't subscribe to %d:%d - %s",
+				port->addr.client, port->addr.port, snd_strerror(res));
+			active = false;
+		}
+		spa_log_info(state->log, "subscribe: %s port %d.%d",
+				port->direction == SPA_DIRECTION_OUTPUT ? "output" : "input",
+				port->addr.client, port->addr.port);
+	} else {
+		if ((res = snd_seq_unsubscribe_port(state->event.hndl, sub)) < 0) {
+			spa_log_warn(state->log, "can't unsubscribe from %d:%d - %s",
+				port->addr.client, port->addr.port, snd_strerror(res));
+		}
+		spa_log_info(state->log, "unsubscribe: %s port %d.%d",
+				port->direction == SPA_DIRECTION_OUTPUT ? "output" : "input",
+				port->addr.client, port->addr.port);
+	}
+	port->active = active;
+	return res;
+}
+
 static struct buffer *peek_buffer(struct seq_state *state,
 		struct seq_port *port)
 {
@@ -418,7 +457,7 @@ static int process_recycle(struct seq_state *state)
 	struct seq_stream *stream = &state->streams[SPA_DIRECTION_OUTPUT];
 	uint32_t i;
 
-	for (i = 0; i < MAX_PORTS; i++) {
+	for (i = 0; i < stream->last_port; i++) {
 		struct seq_port *port = &stream->ports[i];
 		struct spa_io_buffers *io = port->io;
 
@@ -505,7 +544,7 @@ static int process_read(struct seq_state *state)
 	/* prepare a buffer on each port, some ports might have their
 	 * buffer filled above */
 	res = 0;
-	for (i = 0; i < MAX_PORTS; i++) {
+	for (i = 0; i < stream->last_port; i++) {
 		struct seq_port *port = &stream->ports[i];
 		struct spa_io_buffers *io = port->io;
 
@@ -557,7 +596,7 @@ static int process_write(struct seq_state *state)
 	uint32_t i;
 	int res = 0;
 
-	for (i = 0; i < MAX_PORTS; i++) {
+	for (i = 0; i < stream->last_port; i++) {
 		struct seq_port *port = &stream->ports[i];
 		struct spa_io_buffers *io = port->io;
 		struct buffer *buffer;
@@ -630,28 +669,13 @@ static int process_write(struct seq_state *state)
 	return res;
 }
 
-static void init_loop(struct seq_state *state)
-{
-	state->bw = 0.0;
-	state->z1 = state->z2 = state->z3 = 0.0;
-}
-
-static void set_loop(struct seq_state *state, double bw)
-{
-	double w = 2 * M_PI * bw * state->threshold * state->rate.num / state->rate.denom;
-	state->w0 = 1.0 - exp (-20.0 * w);
-	state->w1 = w * 1.5 / state->threshold;
-	state->w2 = w / 1.5;
-	state->bw = bw;
-}
-
 #define NSEC_TO_CLOCK(c,n) ((n) * (c)->rate.denom / ((c)->rate.num * SPA_NSEC_PER_SEC))
 
 static int update_time(struct seq_state *state, uint64_t nsec, bool follower)
 {
 	snd_seq_queue_status_t *status;
 	const snd_seq_real_time_t* queue_time;
-	uint64_t queue_real;
+	uint64_t queue_real, position;
 	double err, corr;
 	uint64_t clock_elapsed, queue_elapsed;
 
@@ -660,6 +684,9 @@ static int update_time(struct seq_state *state, uint64_t nsec, bool follower)
 		state->rate = clock->rate;
 		state->duration = clock->duration;
 		state->threshold = state->duration;
+		position = clock->position;
+	} else {
+		position = 0;
 	}
 
 	/* take queue time */
@@ -670,37 +697,34 @@ static int update_time(struct seq_state *state, uint64_t nsec, bool follower)
 
 	if (state->queue_base == 0) {
 		state->queue_base = nsec - queue_real;
-		state->clock_base = state->position->clock.position;
+		state->clock_base = position;
 	}
 
-	corr = 1.0 - (state->z2 + state->z3);
+	corr = 1.0 - (state->dll.z2 + state->dll.z3);
 
-	clock_elapsed = state->position->clock.position - state->clock_base;
+	clock_elapsed = position - state->clock_base;
 	state->queue_time = nsec - state->queue_base;
 	queue_elapsed = NSEC_TO_CLOCK(state->clock, state->queue_time) / corr;
 
 	err = ((int64_t)clock_elapsed - (int64_t) queue_elapsed);
 
-	if (state->bw == 0.0) {
-		set_loop(state, BW_MAX);
+	if (state->dll.bw == 0.0) {
+		spa_dll_set_bw(&state->dll, SPA_DLL_BW_MAX, state->threshold,
+				state->rate.num / state->rate.denom);
 		state->next_time = nsec;
 		state->base_time = nsec;
 	}
-	state->z1 += state->w0 * (state->w1 * err - state->z1);
-	state->z2 += state->w0 * (state->z1 - state->z2);
-	state->z3 += state->w2 * state->z2;
-
-	corr = 1.0 - (state->z2 + state->z3);
+	corr = spa_dll_update(&state->dll, err);
 
 	if ((state->next_time - state->base_time) > BW_PERIOD) {
 		state->base_time = state->next_time;
-		if (state->bw == BW_MAX)
-			set_loop(state, BW_MED);
-		else if (state->bw == BW_MED)
-			set_loop(state, BW_MIN);
+		if (state->dll.bw > SPA_DLL_BW_MIN)
+			spa_dll_set_bw(&state->dll, state->dll.bw / 2.0,
+					state->threshold, state->rate.num / state->rate.denom);
 
 		spa_log_debug(state->log, NAME" %p: follower:%d rate:%f bw:%f err:%f (%f %f %f)",
-				state, follower, corr, state->bw, err, state->z1, state->z2, state->z3);
+				state, follower, corr, state->dll.bw, err,
+				state->dll.z1, state->dll.z2, state->dll.z3);
 	}
 
 	state->next_time += state->threshold / corr * 1e9 / state->rate.denom;
@@ -726,7 +750,7 @@ int spa_alsa_seq_process(struct seq_state *state)
 
 	res = process_recycle(state);
 
-	if (state->following) {
+	if (state->following && state->position) {
 		update_time(state, state->position->clock.nsec, true);
 		res |= process_read(state);
 	}
@@ -774,13 +798,15 @@ static void reset_buffers(struct seq_state *this, struct seq_port *port)
 		}
 	}
 }
-static void reset_stream(struct seq_state *this, struct seq_stream *stream)
+static void reset_stream(struct seq_state *this, struct seq_stream *stream, bool active)
 {
 	uint32_t i;
-	for (i = 0; i < MAX_PORTS; i++) {
+	for (i = 0; i < stream->last_port; i++) {
 		struct seq_port *port = &stream->ports[i];
-		if (port->valid)
+		if (port->valid) {
 			reset_buffers(this, port);
+			spa_alsa_seq_activate_port(this, port, active);
+		}
 	}
 }
 
@@ -815,8 +841,12 @@ int spa_alsa_seq_start(struct seq_state *state)
 
 	spa_log_debug(state->log, "alsa %p: start follower:%d", state, state->following);
 
-	if ((res = seq_start(state, &state->event)) < 0)
+	if ((res = snd_seq_start_queue(state->event.hndl, state->event.queue_id, NULL)) < 0) {
+		spa_log_error(state->log, "failed to start queue: %s", snd_strerror(res));
 		return res;
+	}
+	while (snd_seq_drain_output(state->event.hndl) > 0)
+		sleep(1);
 
 	if (state->position) {
 		struct spa_io_clock *clock = &state->position->clock;
@@ -825,8 +855,10 @@ int spa_alsa_seq_start(struct seq_state *state)
 		state->threshold = state->duration;
 	}
 
-	reset_stream(state, &state->streams[SPA_DIRECTION_INPUT]);
-	reset_stream(state, &state->streams[SPA_DIRECTION_OUTPUT]);
+	state->started = true;
+
+	reset_stream(state, &state->streams[SPA_DIRECTION_INPUT], true);
+	reset_stream(state, &state->streams[SPA_DIRECTION_OUTPUT], true);
 
 	state->source.func = alsa_on_timeout_event;
 	state->source.data = state;
@@ -836,11 +868,9 @@ int spa_alsa_seq_start(struct seq_state *state)
 	spa_loop_add_source(state->data_loop, &state->source);
 
 	state->queue_base = 0;
-	init_loop(state);
+	spa_dll_init(&state->dll);
 	set_timers(state);
 
-	state->started = true;
-
 	return 0;
 }
 
@@ -889,6 +919,8 @@ static int do_remove_source(struct spa_loop *loop,
 
 int spa_alsa_seq_pause(struct seq_state *state)
 {
+	int res;
+
 	if (!state->started)
 		return 0;
 
@@ -896,9 +928,16 @@ int spa_alsa_seq_pause(struct seq_state *state)
 
 	spa_loop_invoke(state->data_loop, do_remove_source, 0, NULL, 0, true, state);
 
-	seq_stop(state, &state->event);
+	if ((res = snd_seq_stop_queue(state->event.hndl, state->event.queue_id, NULL)) < 0) {
+		spa_log_warn(state->log, "failed to stop queue: %s", snd_strerror(res));
+	}
+	while (snd_seq_drain_output(state->event.hndl) > 0)
+		sleep(1);
 
 	state->started = false;
 
+	reset_stream(state, &state->streams[SPA_DIRECTION_INPUT], false);
+	reset_stream(state, &state->streams[SPA_DIRECTION_OUTPUT], false);
+
 	return 0;
 }
