@@ -233,6 +233,13 @@ typedef struct {
 	uint8_t frequency:4;
 } __attribute__ ((packed)) a2dp_aptx_t;
 
+typedef struct {
+	a2dp_vendor_codec_t info;
+	uint8_t channel_mode:4;
+	uint8_t frequency:4;
+	uint32_t rfa;
+} __attribute__ ((packed)) a2dp_aptx_hd_t;
+
 #elif __BYTE_ORDER == __BIG_ENDIAN
 
 typedef struct {
@@ -326,6 +333,9 @@ struct a2dp_codec {
 	const char *name;
 	const char *description;
 
+	const int send_fill_frames;
+	const int recv_fill_frames;
+
 	int (*fill_caps) (const struct a2dp_codec *codec, uint32_t flags,
 			uint8_t caps[A2DP_MAX_CAPS_SIZE]);
 	int (*select_config) (const struct a2dp_codec *codec, uint32_t flags,
@@ -342,6 +352,8 @@ struct a2dp_codec {
 	int (*get_block_size) (void *data);
 	int (*get_num_blocks) (void *data);
 
+	int (*abr_process) (void *data, size_t unsent);
+
 	int (*start_encode) (void *data,
 		void *dst, size_t dst_size, uint16_t seqnum, uint32_t timestamp);
 	int (*encode) (void *data,
