@@ -59,21 +59,7 @@ int spa_v4l2_open(struct spa_v4l2_device *dev, const char *path)
 
 	spa_log_info(dev->log, "v4l2: Playback device is '%s'", path);
 
-	if (stat(path, &st) < 0) {
-		err = errno;
-		spa_log_error(dev->log, "v4l2: Cannot identify '%s': %d, %s",
-				path, err, strerror(err));
-		goto error;
-	}
-
-	if (!S_ISCHR(st.st_mode)) {
-		spa_log_error(dev->log, "v4l2: %s is no device", path);
-		err = ENODEV;
-		goto error;
-	}
-
 	dev->fd = open(path, O_RDWR | O_NONBLOCK, 0);
-
 	if (dev->fd == -1) {
 		err = errno;
 		spa_log_error(dev->log, "v4l2: Cannot open '%s': %d, %s",
@@ -81,9 +67,22 @@ int spa_v4l2_open(struct spa_v4l2_device *dev, const char *path)
 		goto error;
 	}
 
+	if (fstat(dev->fd, &st) < 0) {
+		err = errno;
+		spa_log_error(dev->log, "v4l2: Cannot identify '%s': %d, %s",
+				path, err, strerror(err));
+		goto error_close;
+	}
+
+	if (!S_ISCHR(st.st_mode)) {
+		spa_log_error(dev->log, "v4l2: %s is no device", path);
+		err = ENODEV;
+		goto error_close;
+	}
+
 	if (xioctl(dev->fd, VIDIOC_QUERYCAP, &dev->cap) < 0) {
 		err = errno;
-		spa_log_error(dev->log, "QUERYCAP: %m");
+		spa_log_error(dev->log, "v4l2: '%s' QUERYCAP: %m", path);
 		goto error_close;
 	}
 	return 0;
@@ -135,7 +134,7 @@ static int spa_v4l2_buffer_recycle(struct impl *this, uint32_t buffer_id)
 
 	if (xioctl(dev->fd, VIDIOC_QBUF, &b->v4l2_buffer) < 0) {
 		err = errno;
-		spa_log_error(this->log, "VIDIOC_QBUF: %m");
+		spa_log_error(this->log, "v4l2: '%s' VIDIOC_QBUF: %m", this->props.device);
 		return -err;
 	}
 
@@ -159,12 +158,11 @@ static int spa_v4l2_clear_buffers(struct impl *this)
 		d = b->outbuf->datas;
 
 		if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_OUTSTANDING)) {
-			spa_log_info(this->log, "v4l2: queueing outstanding buffer %p", b);
+			spa_log_debug(this->log, "v4l2: queueing outstanding buffer %p", b);
 			spa_v4l2_buffer_recycle(this, i);
 		}
 		if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_MAPPED)) {
-			munmap(SPA_MEMBER(b->ptr, -d[0].mapoffset, void),
-					d[0].maxsize - d[0].mapoffset);
+			munmap(b->ptr, d[0].maxsize);
 		}
 		if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_ALLOCATED)) {
 			spa_log_debug(this->log, "v4l2: close %d", (int) d[0].fd);
@@ -567,6 +565,8 @@ spa_v4l2_enum_format(struct impl *this, int seq,
 
 	while (port->next_fmtdesc) {
 		if (filter) {
+			struct v4l2_format fmt;
+
 			video_format = enum_filter_format(filter_media_type,
 					    filter_media_subtype,
 					    filter, port->fmtdesc.index);
@@ -581,13 +581,34 @@ spa_v4l2_enum_format(struct impl *this, int seq,
 				goto next_fmtdesc;
 
 			port->fmtdesc.pixelformat = info->fourcc;
+
+			spa_zero(fmt);
+			fmt.type = port->fmtdesc.type;
+			fmt.fmt.pix.pixelformat = info->fourcc;
+			fmt.fmt.pix.field = V4L2_FIELD_ANY;
+			fmt.fmt.pix.width = 0;
+			fmt.fmt.pix.height = 0;
+
+			if ((res = xioctl(dev->fd, VIDIOC_TRY_FMT, &fmt)) < 0) {
+				spa_log_debug(this->log, "v4l2: '%s' VIDIOC_TRY_FMT %08x: %m",
+						this->props.device, info->fourcc);
+				goto next_fmtdesc;
+			}
+			if (fmt.fmt.pix.pixelformat != info->fourcc) {
+				spa_log_debug(this->log, "v4l2: '%s' VIDIOC_TRY_FMT wanted %.4s gave %.4s",
+						this->props.device, (char*)&info->fourcc,
+						(char*)&fmt.fmt.pix.pixelformat);
+				goto next_fmtdesc;
+			}
+
 		} else {
 			if ((res = xioctl(dev->fd, VIDIOC_ENUM_FMT, &port->fmtdesc)) < 0) {
 				if (errno == EINVAL)
 					goto enum_end;
 
 				res = -errno;
-				spa_log_error(this->log, "VIDIOC_ENUM_FMT: %m");
+				spa_log_error(this->log, "v4l2: '%s' VIDIOC_ENUM_FMT: %m",
+						this->props.device);
 				goto exit;
 			}
 		}
@@ -632,7 +653,8 @@ spa_v4l2_enum_format(struct impl *this, int seq,
 				goto next_fmtdesc;
 
 			res = -errno;
-			spa_log_error(this->log, "VIDIOC_ENUM_FRAMESIZES: %m");
+			spa_log_error(this->log, "v4l2: '%s' VIDIOC_ENUM_FRAMESIZES: %m",
+					this->props.device);
 			goto exit;
 		}
 		if (filter) {
@@ -722,7 +744,8 @@ spa_v4l2_enum_format(struct impl *this, int seq,
 					goto next_frmsize;
 				break;
 			}
-			spa_log_error(this->log, "VIDIOC_ENUM_FRAMEINTERVALS: %m");
+			spa_log_error(this->log, "v4l2: '%s' VIDIOC_ENUM_FRAMEINTERVALS: %m",
+					this->props.device);
 			goto exit;
 		}
 		if (filter) {
@@ -826,7 +849,7 @@ spa_v4l2_enum_format(struct impl *this, int seq,
 	return res;
 }
 
-static int spa_v4l2_set_format(struct impl *this, struct spa_video_info *format, bool try_only)
+static int spa_v4l2_set_format(struct impl *this, struct spa_video_info *format, uint32_t flags)
 {
 	struct port *port = &this->out_ports[0];
 	struct spa_v4l2_device *dev = &port->dev;
@@ -837,6 +860,7 @@ static int spa_v4l2_set_format(struct impl *this, struct spa_video_info *format,
 	uint32_t video_format;
 	struct spa_rectangle *size = NULL;
 	struct spa_fraction *framerate = NULL;
+	bool match;
 
 	spa_zero(fmt);
 	spa_zero(streamparm);
@@ -881,7 +905,7 @@ static int spa_v4l2_set_format(struct impl *this, struct spa_video_info *format,
 	streamparm.parm.capture.timeperframe.numerator = framerate->denom;
 	streamparm.parm.capture.timeperframe.denominator = framerate->num;
 
-	spa_log_info(this->log, "v4l2: set %.4s %dx%d %d/%d", (char *)&fmt.fmt.pix.pixelformat,
+	spa_log_debug(this->log, "v4l2: set %.4s %dx%d %d/%d", (char *)&fmt.fmt.pix.pixelformat,
 		     fmt.fmt.pix.width, fmt.fmt.pix.height,
 		     streamparm.parm.capture.timeperframe.denominator,
 		     streamparm.parm.capture.timeperframe.numerator);
@@ -891,10 +915,11 @@ static int spa_v4l2_set_format(struct impl *this, struct spa_video_info *format,
 	if ((res = spa_v4l2_open(dev, this->props.device)) < 0)
 		return res;
 
-	cmd = try_only ? VIDIOC_TRY_FMT : VIDIOC_S_FMT;
+	cmd = (flags & SPA_NODE_PARAM_FLAG_TEST_ONLY) ? VIDIOC_TRY_FMT : VIDIOC_S_FMT;
 	if (xioctl(dev->fd, cmd, &fmt) < 0) {
 		res = -errno;
-		spa_log_error(this->log, "VIDIOC_S_FMT: %m");
+		spa_log_error(this->log, "v4l2: '%s' VIDIOC_S_FMT: %m",
+				this->props.device);
 		return res;
 	}
 
@@ -902,18 +927,27 @@ static int spa_v4l2_set_format(struct impl *this, struct spa_video_info *format,
 	if (xioctl(dev->fd, VIDIOC_S_PARM, &streamparm) < 0)
 		spa_log_warn(this->log, "VIDIOC_S_PARM: %m");
 
-	spa_log_info(this->log, "v4l2: got %.4s %dx%d %d/%d", (char *)&fmt.fmt.pix.pixelformat,
-		     fmt.fmt.pix.width, fmt.fmt.pix.height,
-		     streamparm.parm.capture.timeperframe.denominator,
-		     streamparm.parm.capture.timeperframe.numerator);
+	match = (reqfmt.fmt.pix.pixelformat == fmt.fmt.pix.pixelformat &&
+			reqfmt.fmt.pix.width == fmt.fmt.pix.width &&
+			reqfmt.fmt.pix.height == fmt.fmt.pix.height);
 
-	if (reqfmt.fmt.pix.pixelformat != fmt.fmt.pix.pixelformat ||
-	    reqfmt.fmt.pix.width != fmt.fmt.pix.width ||
-	    reqfmt.fmt.pix.height != fmt.fmt.pix.height)
+	if (!match && !SPA_FLAG_IS_SET(flags, SPA_NODE_PARAM_FLAG_NEAREST)) {
+		spa_log_error(this->log, "v4l2: wanted %.4s %dx%d, got %.4s %dx%d",
+				(char *)&reqfmt.fmt.pix.pixelformat,
+				reqfmt.fmt.pix.width, reqfmt.fmt.pix.height,
+				(char *)&fmt.fmt.pix.pixelformat,
+				fmt.fmt.pix.width, fmt.fmt.pix.height);
 		return -EINVAL;
+	}
 
-	if (try_only)
-		return 0;
+	if (flags & SPA_NODE_PARAM_FLAG_TEST_ONLY)
+		return match ? 0 : 1;
+
+	spa_log_info(this->log, "v4l2: '%s' got %.4s %dx%d %d/%d",
+			this->props.device, (char *)&fmt.fmt.pix.pixelformat,
+			fmt.fmt.pix.width, fmt.fmt.pix.height,
+			streamparm.parm.capture.timeperframe.denominator,
+			streamparm.parm.capture.timeperframe.numerator);
 
 	dev->have_format = true;
 	size->width = fmt.fmt.pix.width;
@@ -923,13 +957,13 @@ static int spa_v4l2_set_format(struct impl *this, struct spa_video_info *format,
 
 	port->fmt = fmt;
 	port->info.change_mask |= SPA_PORT_CHANGE_MASK_FLAGS | SPA_PORT_CHANGE_MASK_RATE;
-	port->info.flags = (port->export_buf ? SPA_PORT_FLAG_CAN_ALLOC_BUFFERS : 0) |
+	port->info.flags = (port->alloc_buffers ? SPA_PORT_FLAG_CAN_ALLOC_BUFFERS : 0) |
 		SPA_PORT_FLAG_LIVE |
 		SPA_PORT_FLAG_PHYSICAL |
 		SPA_PORT_FLAG_TERMINAL;
 	port->info.rate = SPA_FRACTION(port->rate.num, port->rate.denom);
 
-	return 0;
+	return match ? 0 : 1;
 }
 
 static int query_ext_ctrl_ioctl(struct port *port, struct v4l2_query_ext_ctrl *qctrl)
@@ -944,6 +978,7 @@ static int query_ext_ctrl_ioctl(struct port *port, struct v4l2_query_ext_ctrl *q
 			return res;
 		port->have_query_ext_ctrl = false;
 	}
+	spa_zero(qc);
 	qc.id = qctrl->id;
 	res = xioctl(dev->fd, VIDIOC_QUERYCTRL, &qc);
 	if (res == 0) {
@@ -1055,7 +1090,7 @@ spa_v4l2_enum_controls(struct impl *this, int seq,
 			goto next;
 		}
 		res = -errno;
-		spa_log_error(this->log, "VIDIOC_QUERYCTRL: %m");
+		spa_log_error(this->log, "v4l2: '%s' VIDIOC_QUERYCTRL: %m", this->props.device);
 		return res;
 	}
 	if (result.next & next_fl)
@@ -1207,7 +1242,7 @@ static int mmap_read(struct impl *this)
 	d[0].chunk->stride = port->fmt.fmt.pix.bytesperline;
 	d[0].chunk->flags = 0;
 	if (buf.flags & V4L2_BUF_FLAG_ERROR)
-		d[0].flags |= SPA_CHUNK_FLAG_CORRUPTED;
+		d[0].chunk->flags |= SPA_CHUNK_FLAG_CORRUPTED;
 
 	spa_list_append(&port->queue, &b->link);
 	return 0;
@@ -1222,7 +1257,7 @@ static void v4l2_on_fd_events(struct spa_source *source)
 
 	if (source->rmask & SPA_IO_ERR) {
 		struct port *port = &this->out_ports[0];
-		spa_log_error(this->log, "v4l2 %p: error %08x", this, source->rmask);
+		spa_log_error(this->log, "v4l2: '%p' error %08x", this->props.device, source->rmask);
 		if (port->source.loop)
 			spa_loop_remove_source(this->data_loop, &port->source);
 		return;
@@ -1283,12 +1318,13 @@ static int spa_v4l2_use_buffers(struct impl *this, struct spa_buffer **buffers,
 	reqbuf.count = n_buffers;
 
 	if (xioctl(dev->fd, VIDIOC_REQBUFS, &reqbuf) < 0) {
-		spa_log_error(this->log, "v4l2: VIDIOC_REQBUFS %m");
+		spa_log_error(this->log, "v4l2: '%s' VIDIOC_REQBUFS %m", this->props.device);
 		return -errno;
 	}
-	spa_log_info(this->log, "v4l2: got %d buffers", reqbuf.count);
+	spa_log_debug(this->log, "v4l2: got %d buffers", reqbuf.count);
 	if (reqbuf.count < n_buffers) {
-		spa_log_error(this->log, "v4l2: can't allocate enough buffers");
+		spa_log_error(this->log, "v4l2: '%s' can't allocate enough buffers %d < %d",
+				this->props.device, reqbuf.count, n_buffers);
 		return -ENOMEM;
 	}
 
@@ -1301,7 +1337,7 @@ static int spa_v4l2_use_buffers(struct impl *this, struct spa_buffer **buffers,
 		b->flags = BUFFER_FLAG_OUTSTANDING;
 		b->h = spa_buffer_find_meta_data(buffers[i], SPA_META_Header, sizeof(*b->h));
 
-		spa_log_info(this->log, "v4l2: import buffer %p", buffers[i]);
+		spa_log_debug(this->log, "v4l2: import buffer %p", buffers[i]);
 
 		if (buffers[i]->n_datas < 1) {
 			spa_log_error(this->log, "v4l2: invalid memory on buffer %p", buffers[i]);
@@ -1319,14 +1355,14 @@ static int spa_v4l2_use_buffers(struct impl *this, struct spa_buffer **buffers,
 				void *data;
 
 				data = mmap(NULL,
-					    d[0].maxsize + d[0].mapoffset,
+					    d[0].maxsize,
 					    PROT_READ | PROT_WRITE, MAP_SHARED,
 					    d[0].fd,
-					    0);
+					    d[0].mapoffset);
 				if (data == MAP_FAILED)
 					return -errno;
 
-				b->ptr = SPA_MEMBER(data, d[0].mapoffset, void);
+				b->ptr = data;
 				SPA_FLAG_SET(b->flags, BUFFER_FLAG_MAPPED);
 			}
 			else
@@ -1365,21 +1401,20 @@ mmap_init(struct impl *this,
 	reqbuf.count = n_buffers;
 
 	if (xioctl(dev->fd, VIDIOC_REQBUFS, &reqbuf) < 0) {
-		spa_log_error(this->log, "VIDIOC_REQBUFS: %m");
+		spa_log_error(this->log, "v4l2: '%s' VIDIOC_REQBUFS: %m", this->props.device);
 		return -errno;
 	}
 
-	spa_log_info(this->log, "v4l2: got %d buffers", reqbuf.count);
+	spa_log_debug(this->log, "v4l2: got %d buffers", reqbuf.count);
 	n_buffers = reqbuf.count;
 
-	if (reqbuf.count < 2) {
-		spa_log_error(this->log, "v4l2: can't allocate enough buffers");
+	if (n_buffers < 2) {
+		spa_log_error(this->log, "v4l2: '%s' can't allocate enough buffers (%d)",
+				this->props.device, n_buffers);
 		return -ENOMEM;
 	}
-	if (port->export_buf)
-		spa_log_info(this->log, "v4l2: using EXPBUF");
 
-	for (i = 0; i < reqbuf.count; i++) {
+	for (i = 0; i < n_buffers; i++) {
 		struct buffer *b;
 		struct spa_data *d;
 
@@ -1400,10 +1435,17 @@ mmap_init(struct impl *this,
 		b->v4l2_buffer.index = i;
 
 		if (xioctl(dev->fd, VIDIOC_QUERYBUF, &b->v4l2_buffer) < 0) {
-			spa_log_error(this->log, "VIDIOC_QUERYBUF: %m");
+			spa_log_error(this->log, "v4l2: '%s' VIDIOC_QUERYBUF: %m", this->props.device);
 			return -errno;
 		}
 
+		if (b->v4l2_buffer.flags & V4L2_BUF_FLAG_QUEUED) {
+			/* some drivers can give us an already queued buffer. */
+			spa_log_warn(this->log, "v4l2: buffer %d was already queued", i);
+			n_buffers = i;
+			break;
+		}
+
 		d = buffers[i]->datas;
 		d[0].mapoffset = 0;
 		d[0].maxsize = b->v4l2_buffer.length;
@@ -1412,7 +1454,7 @@ mmap_init(struct impl *this,
 		d[0].chunk->stride = port->fmt.fmt.pix.bytesperline;
 		d[0].chunk->flags = 0;
 
-		if (port->export_buf) {
+		if (port->have_expbuf) {
 			struct v4l2_exportbuffer expbuf;
 
 			spa_zero(expbuf);
@@ -1420,8 +1462,14 @@ mmap_init(struct impl *this,
 			expbuf.index = i;
 			expbuf.flags = O_CLOEXEC | O_RDONLY;
 			if (xioctl(dev->fd, VIDIOC_EXPBUF, &expbuf) < 0) {
-				spa_log_error(this->log, "VIDIOC_EXPBUF: %m");
-				continue;
+				if (errno == ENOTTY || errno == EINVAL) {
+					spa_log_debug(this->log, "v4l2: '%s' VIDIOC_EXPBUF not supported: %m",
+							this->props.device);
+					port->have_expbuf = false;
+					goto fallback;
+				}
+				spa_log_error(this->log, "v4l2: '%s' VIDIOC_EXPBUF: %m", this->props.device);
+				return -errno;
 			}
 			d[0].type = SPA_DATA_DmaBuf;
 			d[0].flags = SPA_DATA_FLAG_READABLE;
@@ -1430,25 +1478,30 @@ mmap_init(struct impl *this,
 			SPA_FLAG_SET(b->flags, BUFFER_FLAG_ALLOCATED);
 			spa_log_debug(this->log, "v4l2: EXPBUF fd:%d", expbuf.fd);
 		} else {
-			d[0].type = SPA_DATA_MemPtr;
+fallback:
+			d[0].type = SPA_DATA_MemFd;
 			d[0].flags = SPA_DATA_FLAG_READABLE;
-			d[0].fd = -1;
+			d[0].fd = dev->fd;
+			d[0].mapoffset = b->v4l2_buffer.m.offset;
 			d[0].data = mmap(NULL,
-					 b->v4l2_buffer.length,
-					 PROT_READ, MAP_SHARED,
-					 dev->fd,
-					 b->v4l2_buffer.m.offset);
+					b->v4l2_buffer.length,
+					PROT_READ, MAP_SHARED,
+					dev->fd,
+					b->v4l2_buffer.m.offset);
 			if (d[0].data == MAP_FAILED) {
-				spa_log_error(this->log, "mmap: %m");
-				continue;
+				spa_log_error(this->log, "v4l2: '%s' mmap: %m", this->props.device);
+				return -errno;
 			}
 			b->ptr = d[0].data;
 			SPA_FLAG_SET(b->flags, BUFFER_FLAG_MAPPED);
-			spa_log_debug(this->log, "v4l2: mmap ptr:%p", d[0].data);
+			spa_log_debug(this->log, "v4l2: mmap offset:%u data:%p", d[0].mapoffset, b->ptr);
 		}
 		spa_v4l2_buffer_recycle(this, i);
 	}
-	port->n_buffers = reqbuf.count;
+	spa_log_info(this->log, "v4l2: have %u buffers using %s", n_buffers,
+			port->have_expbuf ? "EXPBUF" : "MMAP");
+
+	port->n_buffers = n_buffers;
 
 	return 0;
 }
@@ -1507,7 +1560,7 @@ static int spa_v4l2_stream_on(struct impl *this)
 
 	type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 	if (xioctl(dev->fd, VIDIOC_STREAMON, &type) < 0) {
-		spa_log_error(this->log, "VIDIOC_STREAMON: %m");
+		spa_log_error(this->log, "v4l2: '%s' VIDIOC_STREAMON: %m", this->props.device);
 		return -errno;
 	}
 
@@ -1555,7 +1608,7 @@ static int spa_v4l2_stream_off(struct impl *this)
 
 	type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 	if (xioctl(dev->fd, VIDIOC_STREAMOFF, &type) < 0) {
-		spa_log_error(this->log, "VIDIOC_STREAMOFF: %m");
+		spa_log_error(this->log, "v4l2: '%s' VIDIOC_STREAMOFF: %m", this->props.device);
 		return -errno;
 	}
 	for (i = 0; i < port->n_buffers; i++) {
