@@ -78,7 +78,7 @@ struct impl {
 	unsigned int started:1;
 	unsigned int active:1;
 	unsigned int driver:1;
-	unsigned int master:1;
+	unsigned int driving:1;
 	unsigned int monitor:1;
 };
 
@@ -165,7 +165,7 @@ static int link_io(struct impl *this)
 			res, spa_strerror(res));
 	}
 
-	spa_zero(this->io_buffers);
+	this->io_buffers = SPA_IO_BUFFERS_INIT;
 
 	if ((res = spa_node_port_set_io(this->follower,
 			this->direction, 0,
@@ -775,7 +775,7 @@ static int impl_node_process(void *object)
 	if (this->monitor)
 		status |= SPA_STATUS_HAVE_DATA;
 
-	if (this->direction == SPA_DIRECTION_OUTPUT && !this->master) {
+	if (this->direction == SPA_DIRECTION_OUTPUT && !this->driving) {
 		if (this->use_converter)
 			status = spa_node_process(this->convert);
 	}
