@@ -105,6 +105,7 @@ static void handle_device(struct data *data, struct spa_device *device)
 {
 	struct spa_hook listener;
 
+	spa_zero(listener);
 	spa_device_add_listener(device, &listener, &impl_device_events, data);
 
 	while (true) {
@@ -184,6 +185,12 @@ int main(int argc, char *argv[])
 			break;
 		}
 
+		if (factory->version < 1) {
+			printf("factories version %d < %d not supported\n",
+					factory->version, 1);
+			continue;
+		}
+
 		for (iidx = 0;;) {
 			const struct spa_interface_info *info;
 
