@@ -90,13 +90,18 @@ struct impl {
 	struct spa_list device_list;
 };
 
-static struct node *v4l2_find_node(struct device *dev, uint32_t id)
+static struct node *v4l2_find_node(struct device *dev, uint32_t id, const char *name)
 {
 	struct node *node;
+	const char *str;
 
 	spa_list_for_each(node, &dev->node_list, link) {
 		if (node->id == id)
 			return node;
+		if (name != NULL &&
+		    (str = pw_properties_get(node->props, PW_KEY_NODE_NAME)) != NULL &&
+		    strcmp(name, str) == 0)
+			return node;
 	}
 	return NULL;
 }
@@ -117,8 +122,8 @@ static struct node *v4l2_create_node(struct device *dev, uint32_t id,
 {
 	struct node *node;
 	struct impl *impl = dev->impl;
-	int res;
-	const char *str;
+	int i, res;
+	const char *str, *d;
 
 	pw_log_debug("new node %u", id);
 
@@ -145,6 +150,17 @@ static struct node *v4l2_create_node(struct device *dev, uint32_t id,
 		str = "v4l2-device";
 	pw_properties_setf(node->props, PW_KEY_NODE_NAME, "%s.%s", info->factory_name, str);
 
+	for (i = 2; i <= 99; i++) {
+		if ((d = pw_properties_get(node->props, PW_KEY_NODE_NAME)) == NULL)
+			break;
+
+		if (v4l2_find_node(dev, SPA_ID_INVALID, d) == NULL)
+			break;
+
+		pw_properties_setf(node->props, PW_KEY_NODE_NAME, "%s.%s.%d",
+				info->factory_name, str, i);
+	}
+
 	str = pw_properties_get(dev->props, SPA_KEY_DEVICE_DESCRIPTION);
 	if (str == NULL)
 		str = "v4l2-device";
@@ -203,7 +219,7 @@ static void v4l2_device_object_info(void *data, uint32_t id,
 	struct device *dev = data;
 	struct node *node;
 
-	node = v4l2_find_node(dev, id);
+	node = v4l2_find_node(dev, id, NULL);
 
 	if (info == NULL) {
 		if (node == NULL) {
@@ -224,13 +240,18 @@ static const struct spa_device_events v4l2_device_events = {
 	.object_info = v4l2_device_object_info
 };
 
-static struct device *v4l2_find_device(struct impl *impl, uint32_t id)
+static struct device *v4l2_find_device(struct impl *impl, uint32_t id, const char *name)
 {
 	struct device *dev;
+	const char *str;
 
 	spa_list_for_each(dev, &impl->device_list, link) {
 		if (dev->id == id)
 			return dev;
+		if (name != NULL &&
+		    (str = pw_properties_get(dev->props, PW_KEY_DEVICE_NAME)) != NULL &&
+		    strcmp(str, name) == 0)
+			return dev;
 	}
 	return NULL;
 }
@@ -251,6 +272,7 @@ static int v4l2_update_device_props(struct device *dev)
 	struct pw_properties *p = dev->props;
 	const char *s, *d;
 	char temp[32];
+	int i;
 
 	if ((s = pw_properties_get(p, SPA_KEY_DEVICE_NAME)) == NULL) {
 		if ((s = pw_properties_get(p, SPA_KEY_DEVICE_BUS_ID)) == NULL) {
@@ -262,6 +284,18 @@ static int v4l2_update_device_props(struct device *dev)
 	}
 	pw_properties_setf(p, PW_KEY_DEVICE_NAME, "v4l2_device.%s", s);
 
+	for (i = 2; i <= 99; i++) {
+		if ((d = pw_properties_get(p, PW_KEY_DEVICE_NAME)) == NULL)
+			break;
+
+		if (v4l2_find_device(dev->impl, SPA_ID_INVALID,  d) == NULL)
+			break;
+
+		pw_properties_setf(p, PW_KEY_DEVICE_NAME, "v4l2_device.%s.%d", s, i);
+	}
+	if (i == 99)
+		return -EEXIST;
+
 	if (pw_properties_get(p, PW_KEY_DEVICE_DESCRIPTION) == NULL) {
 		d = pw_properties_get(p, PW_KEY_DEVICE_PRODUCT_NAME);
 		if (!d)
@@ -295,9 +329,23 @@ static void device_destroy(void *data)
 	struct node *node;
 
 	pw_log_debug("device %p destroy", device);
+	spa_list_remove(&device->link);
 
 	spa_list_consume(node, &device->node_list, link)
 		v4l2_remove_node(device, node);
+
+	if (device->appeared)
+		spa_hook_remove(&device->device_listener);
+}
+
+static void device_free(void *data)
+{
+	struct device *device = data;
+	pw_log_debug("device %p free", device);
+	spa_hook_remove(&device->listener);
+	pw_unload_spa_handle(device->handle);
+	pw_properties_free(device->props);
+	free(device);
 }
 
 static void device_update(void *data)
@@ -322,11 +370,11 @@ static void device_update(void *data)
 
 static const struct sm_object_events device_events = {
 	SM_VERSION_OBJECT_EVENTS,
-        .destroy = device_destroy,
-        .update = device_update,
+	.destroy = device_destroy,
+	.free = device_free,
+	.update = device_update,
 };
 
-
 static struct device *v4l2_create_device(struct impl *impl, uint32_t id,
 		const struct spa_device_object_info *info)
 {
@@ -401,15 +449,8 @@ exit:
 static void v4l2_remove_device(struct impl *impl, struct device *dev)
 {
 	pw_log_debug("remove device %u", dev->id);
-	spa_list_remove(&dev->link);
-	if (dev->appeared)
-		spa_hook_remove(&dev->device_listener);
 	if (dev->sdevice)
 		sm_object_destroy(&dev->sdevice->obj);
-	spa_hook_remove(&dev->listener);
-	pw_unload_spa_handle(dev->handle);
-	pw_properties_free(dev->props);
-	free(dev);
 }
 
 static void v4l2_udev_object_info(void *data, uint32_t id,
@@ -418,14 +459,14 @@ static void v4l2_udev_object_info(void *data, uint32_t id,
 	struct impl *impl = data;
 	struct device *dev;
 
-	dev = v4l2_find_device(impl, id);
+	dev = v4l2_find_device(impl, id, NULL);
 
 	if (info == NULL) {
 		if (dev == NULL)
 			return;
 		v4l2_remove_device(impl, dev);
 	} else if (dev == NULL) {
-		if ((dev = v4l2_create_device(impl, id, info)) == NULL)
+		if (v4l2_create_device(impl, id, info) == NULL)
 			return;
 	} else {
 		v4l2_update_device(impl, dev, info);
