@@ -295,7 +295,7 @@ on_core_error(void *data, uint32_t id, int seq, int res, const char *message)
   struct core_data *rd = data;
   GstPipeWireDeviceProvider *self = rd->self;
 
-  pw_log_error("error id:%u seq:%d res:%d (%s): %s",
+  pw_log_warn("error id:%u seq:%d res:%d (%s): %s",
           id, seq, res, spa_strerror(res), message);
 
   if (id == PW_ID_CORE) {
@@ -571,6 +571,7 @@ gst_pipewire_device_provider_probe (GstDeviceProvider * provider)
   GST_DEBUG_OBJECT (self, "disconnect");
   pw_proxy_destroy ((struct pw_proxy*)data->registry);
   pw_core_disconnect (self->core);
+  self->core = NULL;
   pw_context_destroy (c);
   pw_loop_destroy (l);
 
@@ -660,6 +661,9 @@ gst_pipewire_device_provider_stop (GstDeviceProvider * provider)
   GstPipeWireDeviceProvider *self = GST_PIPEWIRE_DEVICE_PROVIDER (provider);
 
   GST_DEBUG_OBJECT (self, "stopping provider");
+  if (self->loop)
+    pw_thread_loop_stop (self->loop);
+
   if (self->registry) {
     pw_proxy_destroy ((struct pw_proxy*)self->registry);
     self->registry = NULL;
