@@ -74,7 +74,7 @@ static void *create_object(void *_data,
 	struct pw_impl_client *client = pw_resource_get_client(resource);
 	int res;
 
-	node_resource = pw_resource_new(client, new_id, PW_PERM_RWX, type, version, 0);
+	node_resource = pw_resource_new(client, new_id, PW_PERM_ALL, type, version, 0);
 	if (node_resource == NULL) {
 		res = -errno;
 		goto error_resource;
@@ -97,11 +97,8 @@ error_resource:
 	goto error_exit;
 error_node:
 	pw_log_error("can't create node: %s", spa_strerror(res));
-	pw_resource_errorf(node_resource, res, "can't create node: %s", spa_strerror(res));
-	goto error_exit_free;
-
-error_exit_free:
-	pw_resource_remove(node_resource);
+	pw_resource_errorf_id(resource, new_id, res, "can't create node: %s", spa_strerror(res));
+	goto error_exit;
 error_exit:
 	errno = -res;
 	return NULL;
