@@ -46,13 +46,13 @@
 
 /** \cond */
 
-#define MAX_INPUTS	64
-#define MAX_OUTPUTS	64
+#define MAX_INPUTS	1024
+#define MAX_OUTPUTS	1024
 
 #define MAX_BUFFERS	64
 #define MAX_METAS	16u
 #define MAX_DATAS	64u
-#define MAX_AREAS	1024
+#define MAX_AREAS	2048
 #define MAX_MIX		128
 
 #define CHECK_IN_PORT_ID(this,d,p)       ((d) == SPA_DIRECTION_INPUT && (p) < MAX_INPUTS)
@@ -1122,12 +1122,14 @@ static void node_on_data_fd_events(struct spa_source *source)
 
 	if (source->rmask & SPA_IO_IN) {
 		uint64_t cmd;
+		struct pw_impl_node *node = this->impl->this.node;
 
 		if (SPA_UNLIKELY(spa_system_eventfd_read(this->data_system,
 					this->data_source.fd, &cmd) < 0))
 			pw_log_warn(NAME" %p: read failed %m", this);
 		else if (SPA_UNLIKELY(cmd > 1))
-			pw_log_warn(NAME" %p: missed %"PRIu64" wakeups", this, cmd - 1);
+			pw_log_warn("(%s-%u) client missed %"PRIu64" wakeups",
+				node->name, node->info.id, cmd - 1);
 
 		spa_log_trace_fp(this->log, NAME" %p: got ready", this);
 		spa_node_call_ready(&this->callbacks, SPA_STATUS_HAVE_DATA);
