@@ -267,12 +267,13 @@ static int impl_node_enum_params(void *object, int seq,
 	struct spa_pod_builder b = { 0 };
 	struct spa_result_node_params result;
 	uint32_t count = 0;
+	bool found = false;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 	spa_return_val_if_fail(num != 0, -EINVAL);
 
 	result.id = id;
-	result.next = start;
+	result.next = 0;
 
 	while (true) {
 		struct spa_pod *param;
@@ -286,6 +287,11 @@ static int impl_node_enum_params(void *object, int seq,
 		if (param == NULL || !spa_pod_is_object_id(param, id))
 			continue;
 
+		found = true;
+
+		if (result.index < start)
+			continue;
+
 		spa_pod_builder_init(&b, buffer, sizeof(buffer));
 		if (spa_pod_filter(&b, &result.param, param, filter) != 0)
 			continue;
@@ -296,7 +302,7 @@ static int impl_node_enum_params(void *object, int seq,
 		if (++count == num)
 			break;
 	}
-	return 0;
+	return found ? 0 : -ENOENT;
 }
 
 static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
@@ -317,7 +323,7 @@ static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
 static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 {
 	struct node *this = object;
-	int res;
+	int res = 0;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 
@@ -437,7 +443,8 @@ do_update_port(struct node *this,
 	port = GET_PORT(this, direction, port_id);
 
 	if (!port->valid) {
-		spa_log_info(this->log, "node %p: adding port %d", this, port_id);
+		spa_log_debug(this->log, "node %p: adding port %d, direction %d",
+				this, port_id, direction);
 		port->id = port_id;
 		port->direction = direction;
 		port->have_format = false;
@@ -454,7 +461,7 @@ do_update_port(struct node *this,
 
 		port->have_format = false;
 
-		spa_log_info(this->log, "node %p: port %u update %d params", this, port_id, n_params);
+		spa_log_debug(this->log, "node %p: port %u update %d params", this, port_id, n_params);
 		for (i = 0; i < port->n_params; i++)
 			free(port->params[i]);
 		port->n_params = n_params;
@@ -504,7 +511,7 @@ static void do_uninit_port(struct node *this, enum spa_direction direction, uint
 {
 	struct port *port;
 
-	spa_log_info(this->log, "node %p: removing port %d", this, port_id);
+	spa_log_debug(this->log, "node %p: removing port %d", this, port_id);
 
 	if (direction == SPA_DIRECTION_INPUT) {
 		port = GET_IN_PORT(this, port_id);
@@ -559,6 +566,7 @@ impl_node_port_enum_params(void *object, int seq,
 	struct spa_pod_builder b = { 0 };
 	struct spa_result_node_params result;
 	uint32_t count = 0;
+	bool found = false;
 
 	spa_return_val_if_fail(this != NULL, -EINVAL);
 	spa_return_val_if_fail(num != 0, -EINVAL);
@@ -570,7 +578,7 @@ impl_node_port_enum_params(void *object, int seq,
 			direction, port_id, id, start, num);
 
 	result.id = id;
-	result.next = start;
+	result.next = 0;
 
 	while (true) {
 		struct spa_pod *param;
@@ -584,6 +592,11 @@ impl_node_port_enum_params(void *object, int seq,
 		if (param == NULL || !spa_pod_is_object_id(param, id))
 			continue;
 
+		found = true;
+
+		if (result.index < start)
+			continue;
+
 		spa_pod_builder_init(&b, buffer, sizeof(buffer));
 		if (spa_pod_filter(&b, &result.param, param, filter) < 0)
 			continue;
@@ -594,7 +607,7 @@ impl_node_port_enum_params(void *object, int seq,
 		if (++count == num)
 			break;
 	}
-	return 0;
+	return found ? 0 : -ENOENT;
 }
 
 static int
@@ -637,7 +650,7 @@ impl_node_port_set_io(void *object,
 
 	impl = this->impl;
 
-	spa_log_info(this->log, "node %p: port %d.%d set io %d %p", this,
+	spa_log_debug(this->log, "node %p: port %d.%d set io %d %p", this,
 			direction, port_id, id, data);
 
 	if (id == SPA_IO_Buffers) {
@@ -694,7 +707,7 @@ impl_node_port_use_buffers(void *object,
 	spa_return_val_if_fail(CHECK_PORT(this, direction, port_id), -EINVAL);
 
 	impl = this->impl;
-	spa_log_info(this->log, "node %p: use buffers %p %u", this, buffers, n_buffers);
+	spa_log_debug(this->log, "node %p: use buffers %p %u", this, buffers, n_buffers);
 
 	port = GET_PORT(this, direction, port_id);
 
@@ -1010,7 +1023,7 @@ client_node0_update(void *data,
 		this->info.max_output_ports = max_output_ports;
 	if (change_mask & PW_CLIENT_NODE0_UPDATE_PARAMS) {
 		uint32_t i;
-		spa_log_info(this->log, "node %p: update %d params", this, n_params);
+		spa_log_debug(this->log, "node %p: update %d params", this, n_params);
 
 		for (i = 0; i < this->n_params; i++)
 			free(this->params[i]);
@@ -1024,7 +1037,7 @@ client_node0_update(void *data,
 		spa_node_emit_info(&this->hooks, &this->info);
 	}
 
-	spa_log_info(this->log, "node %p: got node update max_in %u, max_out %u", this,
+	spa_log_debug(this->log, "node %p: got node update max_in %u, max_out %u", this,
 		     this->info.max_input_ports, this->info.max_output_ports);
 }
 
@@ -1041,7 +1054,7 @@ client_node0_port_update(void *data,
 	struct node *this = &impl->node;
 	bool remove;
 
-	spa_log_info(this->log, "node %p: got port update", this);
+	spa_log_debug(this->log, "node %p: got port update", this);
 	if (!CHECK_PORT_ID(this, direction, port_id))
 		return;
 
@@ -1214,7 +1227,8 @@ static void client_node0_resource_destroy(void *data)
 				true,
 				&node->data_source);
 	}
-	pw_impl_node_destroy(this->node);
+	if (this->node)
+		pw_impl_node_destroy(this->node);
 }
 
 static void node_initialized(void *data)
@@ -1247,8 +1261,11 @@ static void node_initialized(void *data)
 static void node_free(void *data)
 {
 	struct impl *impl = data;
+	struct pw_impl_client_node0 *this = &impl->this;
 	struct spa_system *data_system = impl->node.data_system;
 
+	this->node = NULL;
+
 	pw_log_debug("client-node %p: free", &impl->this);
 	node_clear(&impl->node);
 
@@ -1257,6 +1274,9 @@ static void node_free(void *data)
 
 	spa_hook_remove(&impl->node_listener);
 
+	if (this->resource)
+		pw_resource_destroy(this->resource);
+
 	pw_array_clear(&impl->mems);
 
 	if (impl->fds[0] != -1)
