@@ -45,6 +45,7 @@
 #define MAX_FDS 1024
 #define MAX_FDS_MSG 28
 
+#define HDR_SIZE_V0	8
 #define HDR_SIZE	16
 
 static bool debug_messages = 0;
@@ -291,6 +292,8 @@ void pw_protocol_native_connection_destroy(struct pw_protocol_native_connection
 
 	spa_hook_list_call(&conn->listener_list, struct pw_protocol_native_connection_events, destroy, 0);
 
+	spa_hook_list_clean(&conn->listener_list);
+
 	clear_buffer(&impl->out, true);
 	clear_buffer(&impl->in, true);
 	free(impl->out.buffer_data);
@@ -321,16 +324,15 @@ static int prepare_packet(struct pw_protocol_native_connection *conn, struct buf
 		if (p[3] >= 4) {
 			pw_log_warn("old version detected");
 			impl->version = 0;
-			impl->hdr_size = 8;
+			impl->hdr_size = HDR_SIZE_V0;
 		} else {
 			impl->version = 3;
-			impl->hdr_size = 16;
+			impl->hdr_size = HDR_SIZE;
 		}
 		spa_hook_list_call(&conn->listener_list,
 				struct pw_protocol_native_connection_events,
 				start, 0, impl->version);
 	}
-
 	if (impl->version >= 3) {
 		buf->msg.seq = p[2];
 		buf->msg.n_fds = p[3];
@@ -338,6 +340,7 @@ static int prepare_packet(struct pw_protocol_native_connection *conn, struct buf
 		buf->msg.seq = 0;
 		buf->msg.n_fds = 0;
 	}
+
 	data += impl->hdr_size;
 	size -= impl->hdr_size;
 	buf->msg.fds = &buf->fds[buf->fds_offset];
