@@ -41,6 +41,8 @@
 
 #include <pipewire/pipewire.h>
 
+#define DEFAULT_SYSTEM_RUNTIME_DIR "/run/pipewire"
+
 static const char *
 get_remote(const struct spa_dict *props)
 {
@@ -76,33 +78,23 @@ get_runtime_dir(void)
 	return runtime_dir;
 }
 
-int pw_protocol_native_connect_local_socket(struct pw_protocol_client *client,
-					    const struct spa_dict *props,
-					    void (*done_callback) (void *data, int res),
-					    void *data)
+static const char *
+get_system_dir(void)
+{
+	return DEFAULT_SYSTEM_RUNTIME_DIR;
+}
+
+static int try_connect(struct pw_protocol_client *client,
+		const char *runtime_dir, const char *name,
+		void (*done_callback) (void *data, int res),
+		void *data)
 {
 	struct sockaddr_un addr;
 	socklen_t size;
-	const char *runtime_dir, *name;
 	int res, name_size, fd;
-	bool path_is_absolute;
-
-	name = get_remote(props);
-
-	path_is_absolute = name[0] == '/';
-
-	runtime_dir = get_runtime_dir();
 
 	pw_log_info("connecting to '%s' runtime_dir:%s", name, runtime_dir);
 
-	if (runtime_dir == NULL && !path_is_absolute) {
-		pw_log_error("client %p: name %s is not an absolute path and no runtime dir found."
-				"set one of PIPEWIRE_RUNTIME_DIR, XDG_RUNTIME_DIR, HOME or "
-				"USERPROFILE in the environment", client, name);
-		res = -ENOENT;
-		goto error;
-	}
-
 	if ((fd = socket(PF_LOCAL, SOCK_STREAM | SOCK_CLOEXEC | SOCK_NONBLOCK, 0)) < 0) {
 		res = -errno;
 		goto error;
@@ -110,13 +102,13 @@ int pw_protocol_native_connect_local_socket(struct pw_protocol_client *client,
 
 	memset(&addr, 0, sizeof(addr));
 	addr.sun_family = AF_LOCAL;
-	if (!path_is_absolute)
-		name_size = snprintf(addr.sun_path, sizeof(addr.sun_path), "%s/%s", runtime_dir, name) + 1;
-	else
+	if (runtime_dir == NULL)
 		name_size = snprintf(addr.sun_path, sizeof(addr.sun_path), "%s", name) + 1;
+	else
+		name_size = snprintf(addr.sun_path, sizeof(addr.sun_path), "%s/%s", runtime_dir, name) + 1;
 
 	if (name_size > (int) sizeof addr.sun_path) {
-		if (path_is_absolute)
+		if (runtime_dir == NULL)
 			pw_log_error("client %p: socket path \"%s\" plus null terminator exceeds %i bytes",
 				client, name, (int) sizeof(addr.sun_path));
 		else
@@ -148,3 +140,32 @@ error_close:
 error:
 	return res;
 }
+
+int pw_protocol_native_connect_local_socket(struct pw_protocol_client *client,
+					    const struct spa_dict *props,
+					    void (*done_callback) (void *data, int res),
+					    void *data)
+{
+	const char *runtime_dir, *name;
+	int res;
+
+	name = get_remote(props);
+	if (name == NULL)
+		return -EINVAL;
+
+	if (name[0] == '/') {
+		res = try_connect(client, NULL, name, done_callback, data);
+	} else {
+		runtime_dir = get_runtime_dir();
+		if (runtime_dir != NULL) {
+			res = try_connect(client, runtime_dir, name, done_callback, data);
+			if (res >= 0)
+				goto exit;
+		}
+		runtime_dir = get_system_dir();
+		if (runtime_dir != NULL)
+			res = try_connect(client, runtime_dir, name, done_callback, data);
+	}
+exit:
+	return res;
+}
