@@ -30,6 +30,11 @@
 #define FLAG_SEEKMASK			0x000000FFLU
 #define FLAG_SHMWRITABLE		0x00800000LU
 
+#define SEEK_RELATIVE		0
+#define SEEK_ABSOLUTE		1
+#define SEEK_RELATIVE_ON_READ	2
+#define SEEK_RELATIVE_END	3
+
 #define FRAME_SIZE_MAX_ALLOW (1024*1024*16)
 
 #define PROTOCOL_FLAG_MASK	0xffff0000u
