@@ -36,6 +36,7 @@ struct impl {
 	struct spa_hook context_listener;
 	struct pw_array permissions;
 	struct spa_hook pool_listener;
+	unsigned int registered:1;
 };
 
 #define pw_client_resource(r,m,v,...)		pw_resource_call(r,struct pw_client_events,m,v,__VA_ARGS__)
@@ -43,6 +44,7 @@ struct impl {
 #define pw_client_resource_permissions(r,...)	pw_client_resource(r,permissions,0,__VA_ARGS__)
 
 struct resource_data {
+	struct pw_resource *resource;
 	struct spa_hook resource_listener;
 	struct spa_hook object_listener;
 	struct pw_impl_client *client;
@@ -122,8 +124,7 @@ static int error_resource(void *object, void *data)
 
 static int client_error(void *object, uint32_t id, int res, const char *error)
 {
-	struct pw_resource *resource = object;
-	struct resource_data *data = pw_resource_get_user_data(resource);
+	struct resource_data *data = object;
 	struct pw_impl_client *client = data->client;
 	struct error_data d = { id, res, error };
 
@@ -137,12 +138,15 @@ static int update_properties(struct pw_impl_client *client, const struct spa_dic
 	struct pw_resource *resource;
 	int changed = 0;
 	uint32_t i;
+	const char *old;
 
         for (i = 0; i < dict->n_items; i++) {
 		if (filter && strstr(dict->items[i].key, "pipewire.") == dict->items[i].key &&
-		    pw_properties_get(client->properties, dict->items[i].key) != NULL) {
-			pw_log_warn(NAME" %p: refuse property update '%s' to '%s'",
-					client, dict->items[i].key, dict->items[i].value);
+		    (old = pw_properties_get(client->properties, dict->items[i].key)) != NULL &&
+		    (dict->items[i].value == NULL || strcmp(old, dict->items[i].value) != 0)) {
+			pw_log_warn(NAME" %p: refuse property update '%s' from '%s' to '%s'",
+					client, dict->items[i].key, old,
+					dict->items[i].value);
 			continue;
 		}
                 changed += pw_properties_set(client->properties, dict->items[i].key, dict->items[i].value);
@@ -167,18 +171,54 @@ static int update_properties(struct pw_impl_client *client, const struct spa_dic
 	return changed;
 }
 
+static void update_busy(struct pw_impl_client *client)
+{
+	struct pw_permission *def;
+	def = find_permission(client, PW_ID_CORE);
+	pw_impl_client_set_busy(client, (def->permissions & PW_PERM_R) ? false : true);
+}
+
+static int finish_register(struct pw_impl_client *client)
+{
+	struct impl *impl = SPA_CONTAINER_OF(client, struct impl, this);
+	struct pw_impl_client *current;
+	const char *keys[] = {
+		PW_KEY_ACCESS,
+		PW_KEY_CLIENT_ACCESS,
+		PW_KEY_APP_NAME,
+		NULL
+	};
+	if (impl->registered)
+		return 0;
+
+	impl->registered = true;
+
+	current = client->context->current_client;
+	client->context->current_client = NULL;
+	pw_context_emit_check_access(client->context, client);
+	client->context->current_client = current;
+
+	update_busy(client);
+
+	pw_global_update_keys(client->global, client->info.props, keys);
+	pw_global_register(client->global);
+
+	return 0;
+}
+
 static int client_update_properties(void *object, const struct spa_dict *props)
 {
-	struct pw_resource *resource = object;
-	struct resource_data *data = pw_resource_get_user_data(resource);
+	struct resource_data *data = object;
 	struct pw_impl_client *client = data->client;
-	return update_properties(client, props, true);
+	int res = update_properties(client, props, true);
+	finish_register(client);
+	return res;
 }
 
 static int client_get_permissions(void *object, uint32_t index, uint32_t num)
 {
-	struct pw_resource *resource = object;
-	struct resource_data *data = pw_resource_get_user_data(resource);
+	struct resource_data *data = object;
+	struct pw_resource *resource = data->resource;
 	struct pw_impl_client *client = data->client;
 	struct impl *impl = SPA_CONTAINER_OF(client, struct impl, this);
 	size_t len;
@@ -197,8 +237,7 @@ static int client_get_permissions(void *object, uint32_t index, uint32_t num)
 static int client_update_permissions(void *object,
 		uint32_t n_permissions, const struct pw_permission *permissions)
 {
-	struct pw_resource *resource = object;
-	struct resource_data *data = pw_resource_get_user_data(resource);
+	struct resource_data *data = object;
 	struct pw_impl_client *client = data->client;
 	return pw_impl_client_update_permissions(client, n_permissions, permissions);
 }
@@ -213,7 +252,10 @@ static const struct pw_client_methods client_methods = {
 
 static void client_unbind_func(void *data)
 {
-	struct pw_resource *resource = data;
+	struct resource_data *d = data;
+	struct pw_resource *resource = d->resource;
+	spa_hook_remove(&d->resource_listener);
+	spa_hook_remove(&d->object_listener);
 	if (resource->id == 1)
 		resource->client->client_resource = NULL;
 }
@@ -237,13 +279,14 @@ global_bind(void *_data, struct pw_impl_client *client, uint32_t permissions,
 		goto error_resource;
 
 	data = pw_resource_get_user_data(resource);
+	data->resource = resource;
 	data->client = this;
 	pw_resource_add_listener(resource,
 			&data->resource_listener,
-			&resource_events, resource);
+			&resource_events, data);
 	pw_resource_add_object_listener(resource,
 			&data->object_listener,
-			&client_methods, resource);
+			&client_methods, data);
 
 	pw_log_debug(NAME" %p: bound to %d", this, resource->id);
 	pw_global_add_resource(global, resource);
@@ -251,7 +294,7 @@ global_bind(void *_data, struct pw_impl_client *client, uint32_t permissions,
 	if (resource->id == 1)
 		client->client_resource = resource;
 
-	this->info.change_mask = ~0;
+	this->info.change_mask = PW_CLIENT_CHANGE_MASK_ALL;
 	pw_client_resource_info(resource, &this->info);
 	this->info.change_mask = 0;
 
@@ -308,13 +351,6 @@ static const struct pw_context_events context_events = {
 	.global_removed = context_global_removed,
 };
 
-static void update_busy(struct pw_impl_client *client)
-{
-	struct pw_permission *def;
-	def = find_permission(client, PW_ID_CORE);
-	pw_impl_client_set_busy(client, (def->permissions & PW_PERM_R) ? false : true);
-}
-
 /** Make a new client object
  *
  * \param context a \ref pw_context object to register the client with
@@ -386,10 +422,6 @@ struct pw_impl_client *pw_context_create_client(struct pw_impl_core *core,
 
 	this->info.props = &this->properties->dict;
 
-	pw_context_emit_check_access(this->context, this);
-
-	update_busy(this);
-
 	return this;
 
 error_clear_array:
@@ -428,7 +460,6 @@ int pw_impl_client_register(struct pw_impl_client *client,
 		PW_KEY_SEC_UID,
 		PW_KEY_SEC_GID,
 		PW_KEY_SEC_LABEL,
-		PW_KEY_ACCESS,
 		NULL
 	};
 
@@ -452,14 +483,12 @@ int pw_impl_client_register(struct pw_impl_client *client,
 	client->info.id = client->global->id;
 	pw_properties_setf(client->properties, PW_KEY_OBJECT_ID, "%d", client->info.id);
 	client->info.props = &client->properties->dict;
+	pw_global_add_listener(client->global, &client->global_listener, &global_events, client);
 
 	pw_global_update_keys(client->global, client->info.props, keys);
 
 	pw_impl_client_emit_initialized(client);
 
-	pw_global_add_listener(client->global, &client->global_listener, &global_events, client);
-	pw_global_register(client->global);
-
 	return 0;
 
 error_existed:
@@ -547,8 +576,12 @@ void pw_impl_client_destroy(struct pw_impl_client *client)
 	pw_log_debug(NAME" %p: free", impl);
 	pw_impl_client_emit_free(client);
 
+	spa_hook_list_clean(&client->listener_list);
+
 	pw_map_clear(&client->objects);
 	pw_array_clear(&impl->permissions);
+
+	spa_hook_remove(&impl->pool_listener);
 	pw_mempool_destroy(client->pool);
 
 	pw_properties_free(client->properties);
@@ -585,7 +618,9 @@ const struct pw_client_info *pw_impl_client_get_info(struct pw_impl_client *clie
 SPA_EXPORT
 int pw_impl_client_update_properties(struct pw_impl_client *client, const struct spa_dict *dict)
 {
-	return update_properties(client, dict, false);
+	int res = update_properties(client, dict, false);
+	finish_register(client);
+	return res;
 }
 
 SPA_EXPORT
@@ -665,6 +700,7 @@ void pw_impl_client_set_busy(struct pw_impl_client *client, bool busy)
 		pw_impl_client_emit_busy_changed(client, busy);
 	}
 }
+
 SPA_EXPORT
 int pw_impl_client_check_permissions(struct pw_impl_client *client,
 		uint32_t global_id, uint32_t permissions)
