@@ -373,6 +373,7 @@ static void update_info(struct pw_impl_port *port, const struct spa_port_info *i
 					id, spa_debug_type_find_name(spa_type_param, id),
 					port->info.params[i].flags, info->params[i].flags);
 
+			port->info.params[i].id = info->params[i].id;
 			if (port->info.params[i].flags == info->params[i].flags)
 				continue;
 
@@ -1128,7 +1129,6 @@ int pw_impl_port_for_each_param(struct pw_impl_port *port,
 		struct spa_pod_builder b = { 0 };
 	        struct spa_result_node_params result;
 		uint32_t count = 0;
-		bool found = false;
 
 		result.id = param_id;
 		result.next = 0;
@@ -1138,8 +1138,6 @@ int pw_impl_port_for_each_param(struct pw_impl_port *port,
 			if (p->id != param_id)
 				continue;
 
-			found = true;
-
 			if (result.index < index)
 				continue;
 
@@ -1153,7 +1151,7 @@ int pw_impl_port_for_each_param(struct pw_impl_port *port,
 			if (++count == max)
 				break;
 		}
-		res = found ? 0 : -ENOENT;
+		res = 0;
 	} else {
 		user_data.cache = filter == NULL;
 
