@@ -329,6 +329,7 @@ pw_core_resource_errorf(struct pw_resource *resource, uint32_t id, int seq,
 #define pw_context_driver_emit_incomplete(c,n)	pw_context_driver_emit(c, incomplete, 0, n)
 #define pw_context_driver_emit_timeout(c,n)	pw_context_driver_emit(c, timeout, 0, n)
 #define pw_context_driver_emit_drained(c,n)	pw_context_driver_emit(c, drained, 0, n)
+#define pw_context_driver_emit_complete(c,n)	pw_context_driver_emit(c, complete, 0, n)
 
 struct pw_context_driver_events {
 #define PW_VERSION_CONTEXT_DRIVER_EVENTS	0
@@ -344,6 +345,8 @@ struct pw_context_driver_events {
 	void (*timeout) (void *data, struct pw_impl_node *node);
 	/** a node drained */
 	void (*drained) (void *data, struct pw_impl_node *node);
+	/** The driver completed the graph */
+	void (*complete) (void *data, struct pw_impl_node *node);
 };
 
 #define pw_registry_resource(r,m,v,...) pw_resource_call(r, struct pw_registry_events,m,v,##__VA_ARGS__)
@@ -642,6 +645,7 @@ struct pw_impl_node {
 
 	struct pw_loop *data_loop;		/**< the data loop for this node */
 
+	struct spa_fraction latency;		/**< requested latency */
 	uint32_t quantum_size;			/**< desired quantum */
 	struct spa_source source;		/**< source to remotely trigger this node */
 	struct pw_memblock *activation;
