@@ -72,16 +72,15 @@ struct pw_resource *pw_resource_new(struct pw_impl_client *client,
 	}
 
 	if ((res = pw_map_insert_at(&client->objects, id, this)) < 0) {
-		res = -errno;
-		pw_log_error(NAME" %p: can't add id %u for client %p: %m",
-			this, id, client);
+		pw_log_error(NAME" %p: can't add id %u for client %p: %s",
+			this, id, client, spa_strerror(res));
 		goto error_clean;
 	}
 	this->id = id;
 
 	if ((res = pw_resource_install_marshal(this, false)) < 0) {
-		pw_log_error(NAME" %p: no marshal for type %s/%d", this,
-				type, version);
+		pw_log_error(NAME" %p: no marshal for type %s/%d: %s", this,
+				type, version, spa_strerror(res));
 		goto error_clean;
 	}
 
@@ -282,6 +281,9 @@ void pw_resource_destroy(struct pw_resource *resource)
 
 	pw_log_debug(NAME" %p: free %u", resource, resource->id);
 
+	spa_hook_list_clean(&resource->listener_list);
+	spa_hook_list_clean(&resource->object_listener_list);
+
 	free(resource);
 }
 
