@@ -198,6 +198,8 @@ void pw_thread_loop_destroy(struct pw_thread_loop *loop)
 
 	spa_hook_remove(&loop->hook);
 
+	spa_hook_list_clean(&loop->listener_list);
+
 	pw_loop_destroy_source(loop->loop, loop->event);
 
 	if (loop->created)
@@ -238,9 +240,12 @@ static void *do_loop(void *user_data)
 	pw_loop_enter(this->loop);
 
 	while (this->running) {
-		if ((res = pw_loop_iterate(this->loop, -1)) < 0)
+		if ((res = pw_loop_iterate(this->loop, -1)) < 0) {
+			if (res == -EINTR)
+				continue;
 			pw_log_warn(NAME" %p: iterate error %d (%s)",
 					this, res, spa_strerror(res));
+		}
 	}
 	pw_log_debug(NAME" %p: leave thread", this);
 	pw_loop_leave(this->loop);
