@@ -727,9 +727,8 @@ static void on_core_error(void *data, uint32_t id, int seq, int res, const char
 	pw_log_error("error id:%u seq:%d res:%d (%s): %s",
 			id, seq, res, spa_strerror(res), message);
 
-	if (id == 0) {
+	if (id == PW_ID_CORE && res == -EPIPE)
 		pw_main_loop_quit(d->loop);
-	}
 }
 
 static const struct pw_core_events core_events = {
